'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _rxdb = require('rxdb');

var _rxjs = require('rxjs');

var _keyCompression = require('rxdb/plugins/key-compression');

var _keyCompression2 = _interopRequireDefault(_keyCompression);

var _logger = require('./logger');

var _logger2 = _interopRequireDefault(_logger);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

exports.default = {
  rxdb: true,
  prototypes: {},
  overwritable: {
    createKeyCompressor: function createKeyCompressor() {
      var _keyCompression$overw;

      var ans = (_keyCompression$overw = _keyCompression2.default.overwritable).createKeyCompressor.apply(_keyCompression$overw, arguments);

      ans._table = Object.assign({}, ans.table, {
        rx_model: 'rx_model'
      });

      return ans;
    }
  },
  hooks: {
    createRxDatabase: function createRxDatabase(database) {
      database.replications = [];
      database.replicate = function replicate() {
        for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
          args[_key] = arguments[_key];
        }

        var replication = new (Function.prototype.bind.apply(Replication, [null].concat([database.collections], args)))();

        database.replications.push(replication);
        var index = database.replications.length - 1;
        replication.destroy = function () {
          var _ref = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee() {
            return regeneratorRuntime.wrap(function _callee$(_context) {
              while (1) {
                switch (_context.prev = _context.next) {
                  case 0:
                    _context.next = 2;
                    return replication.close();

                  case 2:
                    database.replications = database.replications.slice(0, index).concat(database.replications.slice(index + 1));

                  case 3:
                  case 'end':
                    return _context.stop();
                }
              }
            }, _callee, this);
          }));

          function destroy() {
            return _ref.apply(this, arguments);
          }

          return destroy;
        }();

        return replication;
      };
    },
    preCreateRxCollection: function preCreateRxCollection(model) {
      var name = model.name;
      if (!name) throw Error('RxCollection(s) must have a "name" property');
      if (!model.schema || !model.schema.properties) {
        throw Error('RxCollection(s) must have a a "schema" property, with a "properties" key');
      }

      var rxModel = model.schema.properties.rx_model;
      if (rxModel && (rxModel.type !== 'string' || rxModel.default !== name)) {
        throw Error('Schema properties cannot be called "rx_model"');
      }
      model.schema.properties.rx_model = {
        type: 'string',
        enum: [name],
        default: name
      };
    }
  }
};

var Replication = function () {
  function Replication(collections, remote, collectionNames, direction) {
    var options = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : {};

    _classCallCheck(this, Replication);

    this.remote = remote;
    this.directon = direction;
    this.options = options;
    this.collections = !collectionNames ? collections : collectionNames.reduce(function (acc, key) {
      if (collections[key]) acc[key] = collections[key];
      return acc;
    }, {});

    this.replicationStates = [];
    this._pReplicationStates = Promise.resolve([]);
    this._subscribers = [];
    this._states = [];
    this.alive = false;
    this._aliveSubject = new _rxjs.BehaviorSubject(false);
  }

  _createClass(Replication, [{
    key: 'connect',
    value: function () {
      var _ref2 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee2() {
        var _this = this;

        return regeneratorRuntime.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                _context2.next = 2;
                return this.close();

              case 2:
                _context2.prev = 2;
                _context2.next = 5;
                return this._createFilter(this.remote);

              case 5:
                _context2.next = 7;
                return this._sync();

              case 7:
                return _context2.abrupt('return', true);

              case 10:
                _context2.prev = 10;
                _context2.t0 = _context2['catch'](2);

                _logger2.default.error(_context2.t0);
                this._interval = setInterval(function () {
                  _this._createFilter(_this.remote).then(function () {
                    clearInterval(_this._interval);
                    _this._sync();
                  }).catch(function (e) {
                    return _logger2.default.error(e);
                  });
                }, 5000);
                return _context2.abrupt('return', false);

              case 15:
              case 'end':
                return _context2.stop();
            }
          }
        }, _callee2, this, [[2, 10]]);
      }));

      function connect() {
        return _ref2.apply(this, arguments);
      }

      return connect;
    }()
  }, {
    key: 'close',
    value: function () {
      var _ref3 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee3() {
        return regeneratorRuntime.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                clearInterval(this._interval);

                this._subscribers.forEach(function (x) {
                  return x.unsubscribe();
                });
                this._subscribers = [];
                this._states = [];

                if (this.alive) {
                  this.alive = false;
                  this._aliveSubject.next(false);
                }

                _context3.next = 7;
                return this._pReplicationStates.then(function (arr) {
                  return Promise.all(arr.map(function (x) {
                    return x.cancel();
                  }));
                });

              case 7:
                this._pReplicationStates = Promise.resolve([]);
                this.replicationStates = [];

              case 9:
              case 'end':
                return _context3.stop();
            }
          }
        }, _callee3, this);
      }));

      function close() {
        return _ref3.apply(this, arguments);
      }

      return close;
    }()
    // Private

  }, {
    key: '_sync',
    value: function () {
      var _ref4 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee4() {
        var _this2 = this;

        var collections, collectionNames, promises, allAlive;
        return regeneratorRuntime.wrap(function _callee4$(_context4) {
          while (1) {
            switch (_context4.prev = _context4.next) {
              case 0:
                collections = this.collections;
                collectionNames = Object.keys(collections);
                promises = collectionNames.map(function (name) {
                  return collections[name].sync({
                    remote: _this2.remote,
                    direction: _this2.direction,
                    options: Object.assign({}, _this2.options, {
                      live: _this2.options.live || true,
                      retry: _this2.options.retry || true,
                      // selector: { rx_model: name }
                      filter: 'app/by_model',
                      query_params: { rx_model: name }
                    })
                  });
                });
                allAlive = promises.map(function () {
                  return false;
                });

                this._pReplicationStates = Promise.all(promises).then(function (arr) {
                  arr.forEach(function (rep, i) {
                    _this2._subscribers.push(rep.alive$.subscribe(function (val) {
                      var repAlive = allAlive[i];

                      if (repAlive === val) return;

                      allAlive[i] = val;
                      var alive = allAlive.reduce(function (acc, x) {
                        return acc && x;
                      }, true);

                      if (alive === _this2.alive) return;
                      _this2.alive = alive;
                      _this2._aliveSubject.next(alive);
                    }));
                  });
                  return arr;
                }).then(function (arr) {
                  return _this2.replicationStates = arr;
                });

                _context4.next = 7;
                return this._pReplicationStates;

              case 7:
              case 'end':
                return _context4.stop();
            }
          }
        }, _callee4, this);
      }));

      function _sync() {
        return _ref4.apply(this, arguments);
      }

      return _sync;
    }()
  }, {
    key: '_createFilter',
    value: function () {
      var _ref5 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee5() {
        var remoteIsUrl, db, doc;
        return regeneratorRuntime.wrap(function _callee5$(_context5) {
          while (1) {
            switch (_context5.prev = _context5.next) {
              case 0:
                // https://pouchdb.com/2015/04/05/filtered-replication.html
                remoteIsUrl = typeof this.remote === 'string';
                db = remoteIsUrl ? new _rxdb.PouchDB(this.remote) : this.remote;
                doc = {
                  version: 0,
                  _id: '_design/app',
                  filters: {
                    // not doing fn.toString() as istambul code
                    // on tests breaks it
                    by_model: 'function(doc, req) {\n          return (\n            doc._id === \'_design/app\' || doc.rx_model === req.query.rx_model\n          );\n        }'
                  }
                };
                _context5.next = 5;
                return db.get('_design/app').then(function (_ref6) {
                  var version = _ref6.version,
                      _rev = _ref6._rev;

                  return version < doc.version ? db.put(Object.assign({}, doc, { _rev: _rev })) : true;
                }).catch(function () {
                  return db.put(doc);
                });

              case 5:

                if (remoteIsUrl) db.close();

              case 6:
              case 'end':
                return _context5.stop();
            }
          }
        }, _callee5, this);
      }));

      function _createFilter() {
        return _ref5.apply(this, arguments);
      }

      return _createFilter;
    }()
  }, {
    key: 'alive$',
    get: function get() {
      return this._aliveSubject.asObservable();
    }
  }]);

  return Replication;
}();