'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = RxDocument;

var _rxjs = require('rxjs');

var Rx = _interopRequireWildcard(_rxjs);

var _operators = require('rxjs/operators');

var _parseProperties = require('./parse-properties');

var _parseProperties2 = _interopRequireDefault(_parseProperties);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function RxDocument(proto) {
  Object.assign(proto, {
    select$: function select$() {
      var _this = this;

      for (var _len = arguments.length, properties = Array(_len), _key = 0; _key < _len; _key++) {
        properties[_key] = arguments[_key];
      }

      properties = (0, _parseProperties2.default)(properties);

      // Properties not specified. Get all first level properties,
      // including views from views plugin (if they exist)
      if (!Object.keys(properties).length) {
        var viewsKeys = this.collection._views;
        return viewsKeys && viewsKeys.length ? Rx.combineLatest.apply(Rx, [this.$].concat(_toConsumableArray(viewsKeys.map(function (key) {
          return _this[key].$;
        })))).pipe((0, _operators.map)(function (all) {
          return Object.assign({}, all[0], viewsKeys.reduce(function (acc, key, i) {
            acc[key] = all[i + 1];
            return acc;
          }, { _: _this }));
        })) : this.$.pipe((0, _operators.map)(function (res) {
          return Object.assign({}, res, { _: _this });
        }));
      }

      // Properties specified, parse
      return createCombinedObservable(Rx.of(this), properties);
    }
  });
}

function createCombinedObservable(obs, properties) {
  var keys = Object.keys(properties);

  var getObs = [];
  return obs.pipe((0, _operators.switchMap)(function (res) {
    if (!getObs.length) {
      keys.forEach(function (key) {
        if (res[key + '$']) return getObs.push(function (x) {
          return x[key + '$'];
        });

        var value = res[key];
        if (value && value.$) return getObs.push(function (x) {
          return x[key].$;
        });

        return getObs.push(function (x) {
          return x[key];
        });
      });
    }

    return Rx.combineLatest.apply(Rx, _toConsumableArray(getObs.map(function (getObsFn, i) {
      var nextProperties = properties[keys[i]];
      return Object.keys(nextProperties).length ? createCombinedObservable(getObsFn(res), nextProperties) : getObsFn(res);
    }))).pipe((0, _operators.map)(function (all) {
      return all.reduce(function (acc, val, i) {
        acc[keys[i]] = val;
        return acc;
      }, { _: res });
    }));
  }));
}