'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = createSubject;

var _rxjs = require('rxjs');

var _constants = require('./constants');

function createSubject(observable) {
  var opts = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

  var options = {
    keepOpenCheck: function keepOpenCheck() {
      return false;
    },
    onInit: function onInit() {},
    onTeardown: function onTeardown() {}
  };
  Object.assign(options, opts);

  var subject = void 0;
  var subscription = void 0;
  var lifecycle = {
    init: function init() {
      subject = new _rxjs.ReplaySubject(1);
      subscription = observable.subscribe(subject);
      options.onInit();
    },
    teardown: function teardown() {
      var subs = subscription;
      var subj = subject;
      options.onTeardown();
      subs.unsubscribe();
      subj.unsubscribe();
    },
    subscribe: function subscribe(obs) {
      return subject.subscribe(obs);
    },
    unsubscribe: function unsubscribe(subs) {
      subs.unsubscribe();
    }
  };

  var subscriptions = 0;
  var isAlive = false;
  var interval = void 0;
  return _rxjs.Observable.create(function (obs) {
    subscriptions++;
    clearInterval(interval);
    if (!isAlive) (isAlive = true) && lifecycle.init();

    var subs = lifecycle.subscribe(obs);

    return function () {
      subscriptions--;
      lifecycle.unsubscribe(subs);

      if (subscriptions) return;

      setTimeout(function () {
        if (subscriptions || !isAlive) return;

        var kill = function kill() {
          isAlive = false;
          lifecycle.teardown();
          clearInterval(interval);
        };

        if (!options.keepOpenCheck()) kill();else {
          clearInterval(interval);
          interval = setInterval(function () {
            if (subscriptions || !isAlive) return clearInterval(interval);
            if (!options.keepOpenCheck()) kill();
          }, _constants.CHECK_KEEP_OPEN_TIMEOUT);
        }
      }, _constants.CLOSE_SUBSCRIPTION_TIMEOUT);
    };
  });
}