'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = RxQuery;

var _rxjs = require('rxjs');

var Rx = _interopRequireWildcard(_rxjs);

var _operators = require('rxjs/operators');

var _lodash = require('lodash.flattendeep');

var _lodash2 = _interopRequireDefault(_lodash);

var _objectHash = require('object-hash');

var _objectHash2 = _interopRequireDefault(_objectHash);

var _createSubject = require('./create-subject');

var _createSubject2 = _interopRequireDefault(_createSubject);

var _constants = require('./constants');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function RxQuery(proto) {
  Object.assign(proto, {
    ensure$: function ensure$() {
      for (var _len = arguments.length, properties = Array(_len), _key = 0; _key < _len; _key++) {
        properties[_key] = arguments[_key];
      }

      if (!this.collection.options || !this.collection.options.views) {
        throw Error('RxQuery.ensure$() can only be used on collection with defined views');
      }
      properties = (0, _lodash2.default)(properties);
      if (!properties.length) properties = this.collection._views;

      var ensureHash = (0, _objectHash2.default)(properties, { unorderedArrays: true });

      var ensures = this[_constants.ENSURE_SYMBOL];
      if (!ensures) ensures = this[_constants.ENSURE_SYMBOL] = {};

      var obs = ensures[ensureHash];
      if (!obs) obs = ensures[ensureHash] = createEnsure(this.$, properties);

      return obs;
    }
  });
}

function createEnsure(observable, properties) {
  var unensureFns = {};
  var lastDocs = void 0;

  var obs = observable.pipe((0, _operators.switchMap)(function (ans) {
    if (!ans) return Rx.of(ans);

    var docs = Array.isArray(ans) ? ans : [ans];
    var rxArr = [];
    lastDocs = docs;

    docs.forEach(function (doc) {
      var primary = doc.primary;
      if (unensureFns[primary]) return;

      var unensure = doc[_constants.ENSURE_SYMBOL](properties);
      unensureFns[primary] = unensure;
      properties.forEach(function (property) {
        rxArr.push(doc[property].$);
      });
    });

    return rxArr.length ? Rx.combineLatest.apply(Rx, rxArr).pipe((0, _operators.take)(1), (0, _operators.map)(function () {
      return ans;
    })) : Rx.of(ans);
  }));

  var interval = void 0;
  var lastCleanupDocs = void 0;

  return (0, _createSubject2.default)(obs, {
    onInit: function onInit() {
      // Periodic cleanup of ensured docs that are not part of the
      // query response anymore
      interval = setInterval(function () {
        if (!lastDocs || lastCleanupDocs === lastDocs) return;

        lastCleanupDocs = lastDocs;
        var primaries = lastCleanupDocs.reduce(function (acc, doc) {
          acc[doc.primary] = true;
          return acc;
        }, {});

        Object.keys(unensureFns).forEach(function (key) {
          if (primaries[key]) return;

          unensureFns[key]();
          delete unensureFns[key];
        });
      }, _constants.ENSURE_CLEANUP_TIMEOUT);
    },
    onTeardown: function onTeardown() {
      var fns = unensureFns;
      unensureFns = {};
      lastDocs = null;
      lastCleanupDocs = null;
      clearInterval(interval);
      Object.values(fns).forEach(function (fn) {
        return fn();
      });
    }
  });
}